'use strict';

var keypress = require('keypress');

var _require = require('../store'),
    getUiState = _require.getUiState;

keypress(process.stdin);

var buffer = '';
var history = [];
var currentSelectedHistory = 0;

var onCtrlC = function onCtrlC() {
	buffer = '';
	process.exit();
};

var onBackspace = function onBackspace() {
	buffer = buffer.substring(0, buffer.length - 1);
	getUiState().onChange(buffer);
};

var onUp = function onUp() {
	var uiState = getUiState();
	if (history[currentSelectedHistory]) {
		buffer = history[currentSelectedHistory];
		uiState.onChange(buffer);
	} else {
		buffer = '';
		uiState.onChange(buffer);
	}

	if (currentSelectedHistory >= history.length) {
		currentSelectedHistory = 0;
	} else {
		currentSelectedHistory += 1;
	}
};

var onDown = function onDown() {
	var uiState = getUiState();
	if (currentSelectedHistory >= history.length - 1) {
		currentSelectedHistory = history.length - 2;
	} else if (currentSelectedHistory <= -1) {
		currentSelectedHistory = -1;
	} else {
		currentSelectedHistory -= 1;
	}

	if (history[currentSelectedHistory]) {
		buffer = history[currentSelectedHistory];
		uiState.onChange(buffer);
	} else {
		buffer = '';
		uiState.onChange(buffer);
	}
};

var onReturn = function onReturn(onCommandEntered) {
	if (buffer.length > 0) {
		onCommandEntered(buffer.trim().replace(/\n/g, '').replace(/\t/g, ''));
		history.reverse();
		history.push(buffer);
		history.reverse();
	}
	buffer = '';
	currentSelectedHistory = 0;
	getUiState().onChange('');
};

var onRecieveLetter = function onRecieveLetter(letter) {
	if (!letter) {
		return;
	}
	buffer += letter;
	getUiState().onChange(buffer);
};

var keyNameMap = {
	backspace: onBackspace,
	up: onUp,
	down: onDown,
	return: onReturn
};

/**
 * @param {Function<string>} onCommandEntered - the callback function
 * @returns {void}
 **/
function commandListener(onCommandEntered) {
	process.stdin.setEncoding('utf8');
	process.stdin.setRawMode(true);
	process.stdin.on('keypress', function (letter, key) {
		if (!key && letter) {
			onRecieveLetter(letter);
			return;
		}

		if (key) {
			if (key.ctrl && key.name) {
				onCtrlC();
				return;
			}

			/* istanbul ignore else case */
			if (keyNameMap[key.name]) {
				keyNameMap[key.name](onCommandEntered);
				return;
			}
		}
		onRecieveLetter(letter);
	});
	process.stdin.resume();
}

module.exports = commandListener;