/* eslint no-console: 0*/
/* eslint complexity: 0*/
/* eslint no-empty: 0*/
'use strict';

var spawn = require('cross-spawn');

/**
 * @description run the given script in childProcess
 * @param {Object} options - the options
 * @param {string} options.scriptName - the name of the script which should be executed
 * @param {string} options.packagePath - the path in which the given script should be executed
 * @param {Function} options.onExit - callback which will be fired if childProcess was killed
 * @param {Function} options.onRecieve - callback which will be fired if childProcess recieved an message
 * @param {Function} options.onError - callback which will be fired if childProcess recieved an error message
 * @param {Function} options.onError - callback which will be fired if childProcess recieved an error message
 * @returns {Object} returns an object including an start and stop method
 **/
function runNpmScript(_ref) {
	var scriptName = _ref.scriptName,
	    packagePath = _ref.packagePath,
	    _ref$onExit = _ref.onExit,
	    onExit = _ref$onExit === undefined ? function () {} : _ref$onExit,
	    _ref$onRecieve = _ref.onRecieve,
	    onRecieve = _ref$onRecieve === undefined ? function () {} : _ref$onRecieve,
	    _ref$onError = _ref.onError,
	    onError = _ref$onError === undefined ? function () {} : _ref$onError;

	var run = function run(cmd) {
		var cmdArr = cmd.split(' ');

		var childProcessObj = { isRunning: true };

		var childProcess = spawn(cmdArr[0], cmdArr.slice(1, cmdArr.length), {
			shell: true,
			cwd: packagePath,
			detached: true
		});

		childProcess.stdout.on('data', function (data) {
			onRecieve(data.toString().replace(/\n$/, ''));
		});
		childProcess.stderr.on('data', function (data) {
			onError(data.toString().replace(/\n$/, ''));
		});
		childProcess.on('exit', function () {
			childProcessObj.isRunning = false;
			onExit();
		});

		return Object.assign(childProcessObj, {
			stop: function stop() {
				childProcessObj.isRunning = false;
				try {
					process.kill(-childProcess.pid);
				} catch (err) {}
			},
			start: function start() {
				this.stop();
				return run('npm run ' + scriptName);
			},
			execute: function execute(newCmd) {
				this.stop();
				return run(newCmd);
			}
		});
	};

	return run('npm run ' + scriptName);
}

module.exports = runNpmScript;